/*
 * Copyright 2016 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file start_ceph.c
 * Implementation of the Ceph DSI start session
 */
#include "globus_i_ceph.h"
#include "rgadmin.h"

typedef struct
{
    globus_gfs_operation_t              op;
    globus_gfs_session_info_t          *session_info;
    int                                 retries;
    S3Status                            status;
    globus_result_t                     result;
    globus_i_s3_handle_t               *s3_handle;
}
globus_l_ceph_start_t;

static
void *
globus_l_ceph_start_thread(
    void                               *user_arg);

static
void
globus_l_ceph_start_complete(
    S3Status                            status,
    const S3ErrorDetails               *error_details,
    void                               *callback_data);

static
S3Status
globus_l_ceph_start_properties(
    const S3ResponseProperties         *properties,
    void                               *callback_arg);

static
S3Status
globus_l_ceph_get_user_info(
    const RGGetUserInfo                *user_info,
    void                               *callback_data);

/**
 * @brief start
 * @details
 * This function is called when a new session is initialized, ie a user 
 * connects to the server.  This hook gives the dsi an opportunity to
 * reject the user, or set internal state that will be threaded through to all
 * other function calls associated with this session.
 *
 * The value of 'finished_info.info.session.session_arg' should be set to an
 * DSI defined data structure.  This pointer will be passed as the void *
 * user_arg parameter to all other interface functions.
 */
void
globus_i_ceph_start(
    globus_gfs_operation_t              op,
    globus_gfs_session_info_t          *session_info)
{
    globus_l_ceph_start_t              *start_state = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_thread_t                     t;
    int                                 rc;

    start_state = malloc(sizeof(globus_l_ceph_start_t));
    if (start_state == NULL)
    {
        result = GlobusGFSErrorMemory("start_state");
        goto malloc_start_state_fail;
    }

    *start_state = (globus_l_ceph_start_t)
    {
        .op                           = op,
        .session_info                 = session_info
    };

    rc = globus_thread_create(
            &t,
            NULL,
            globus_l_ceph_start_thread,
            start_state);

    if (rc != GLOBUS_SUCCESS)
    {
        result = GlobusGFSErrorGeneric("globus_thread_create");
    }

    if (result != GLOBUS_SUCCESS)
    {
        free(start_state);
malloc_start_state_fail:

        globus_gridftp_server_finished_session_start(
                op, result, NULL, NULL, NULL);
    }

    GlobusS3Exit();

    return;
}
/* globus_i_ceph_start() */

static
void *
globus_l_ceph_start_thread(
    void                               *user_arg)
{
    globus_l_ceph_start_t              *start_state = user_arg;

    GlobusS3Enter();

    GlobusS3Debug("Looking up key for %s\n",
		start_state->session_info->username);

    do
    {
        start_state->status = S3StatusOK;
        start_state->result = GLOBUS_SUCCESS;

        globus_i_s3_retry_delay(start_state->retries);

        GlobusS3Info("REST: RG_get_user_info(user=%s) try=%d\n",
                 (void *) start_state->session_info->username,
                 start_state->retries);

        RG_get_user_info(
                &(S3BucketContext)
                {
                    .bucketName           = globus_i_s3_rg_admin_bucket
                            ? globus_i_s3_rg_admin_bucket
                            : "admin",
                    .protocol             = globus_i_s3_https
                            ? S3ProtocolHTTPS
                            : S3ProtocolHTTP ,
                    .uriStyle             = S3UriStylePath,
                    .accessKeyId          = globus_i_s3_rg_admin_access_key_id,
                    .secretAccessKey      = globus_i_s3_rg_admin_secret_access_key
                },
                start_state->session_info->username,
                NULL,
                &(RGGetUserInfoHandler)
                {
                    .responseHandler.propertiesCallback
                                          = globus_l_ceph_start_properties,
                    .responseHandler.completeCallback
                                          = globus_l_ceph_start_complete,
                    .getUserInfoCallback  = globus_l_ceph_get_user_info
                },
                start_state);
    } while (S3_status_is_retryable(start_state->status)
            && (start_state->retries++) < globus_i_s3_max_retries);

    if (start_state->s3_handle == NULL ||
        start_state->s3_handle->accessKeyId == NULL ||
        start_state->s3_handle->secretAccessKey == NULL)
    {
        const char *fmt = "No access key for ceph user \"%s\"";
        size_t msglen = snprintf(NULL, 0, fmt, start_state->session_info->username);
        char msg[msglen+1];
        snprintf(msg, sizeof(msg), fmt, start_state->session_info->username);

        start_state->result = GlobusGFSErrorGeneric(msg);
    }
    GlobusS3Info("finished_session_start: op=%p result=%#x\n",
            start_state->op,
            start_state->result);

    globus_gridftp_server_finished_session_start(
            start_state->op,
            start_state->result, 
            start_state->s3_handle,
            start_state->session_info->username,
            "/");

    free(start_state);

    return NULL;
}
/* globus_l_ceph_start_thread() */

static
S3Status
globus_l_ceph_start_properties(
    const S3ResponseProperties         *properties,
    void                               *callback_arg)
{
    GlobusS3Enter();
    /* Don't care */
    GlobusS3Exit();
    return S3StatusOK;
}
/* globus_l_ceph_start_properties() */

static
void
globus_l_ceph_start_complete(
    S3Status                            status,
    const S3ErrorDetails               *error_details,
    void                               *callback_data)
{
    globus_l_ceph_start_t                *start_state = callback_data;

    GlobusS3Enter();

    if (status != S3StatusOK)
    {
        start_state->status = status;
        start_state->result = GlobusGFSErrorS3(status, "admin/user", error_details);
    }
    GlobusS3Exit();
}
/* globus_l_ceph_start_complete() */

static
S3Status
globus_l_ceph_get_user_info(
    const RGGetUserInfo                *user_info,
    void                               *callback_data)
{
    S3Status                            status = S3StatusOK;
    globus_l_ceph_start_t                *start_state = callback_data;

    GlobusS3Enter();

    start_state->s3_handle = malloc(sizeof(globus_i_s3_handle_t));
    if (start_state->s3_handle == NULL)
    {
        status = S3StatusOutOfMemory;
        start_state->result = GlobusGFSErrorMemory("s3_handle");
        goto done;
    }
    *start_state->s3_handle = (globus_i_s3_handle_t)
    {
        .protocol                     = globus_i_s3_https
                                        ? S3ProtocolHTTPS
                                        : S3ProtocolHTTP,
        .uriStyle                     = globus_i_s3_use_virtual_host
                                        ? S3UriStyleVirtualHost
                                        : S3UriStylePath,
    };

    for (size_t i = 0; i < user_info->keysCount; i++)
    {
        if (strcmp(user_info->keys[i].user, start_state->session_info->username) == 0)
        {
            if (user_info->keys[i].accessKeyId[0] != 0 &&
                user_info->keys[i].secretAccessKey[0] != 0)
            {
                start_state->s3_handle->accessKeyId = strdup(
                        user_info->keys[i].accessKeyId);
                if (start_state->s3_handle->accessKeyId == NULL)
                {
                    status = S3StatusOutOfMemory;
                    start_state->result = GlobusGFSErrorMemory("accessKeyId");
                    goto done;
                }
                start_state->s3_handle->secretAccessKey = strdup(
                        user_info->keys[i].secretAccessKey);
                if (start_state->s3_handle->secretAccessKey == NULL)
                {
                    status = S3StatusOutOfMemory;
                    start_state->result = GlobusGFSErrorMemory("secretAccessKey");
                    goto done;
                }
                break;
            }
        }
    }
done:
    GlobusS3ExitStatus(status);
    return status;
}
/* globus_l_ceph_get_user_info() */
// vim: ft=c:
